@@ -1,17 +1,15 @@
-from __future__ import division
 from collections import namedtuple
 import numpy as np
 from mpl_toolkits.axes_grid1 import make_axes_locatable
 import matplotlib.image
 from ...util import dtype as dtypes
 from ...exposure import is_low_contrast
-from ...util.colormap import viridis
 from ..._shared.utils import warn
 from math import floor, ceil
 
 
 _default_colormap = 'gray'
-_nonstandard_colormap = viridis
+_nonstandard_colormap = 'viridis'
 _diverging_colormap = 'RdBu'
 
 
@@ -148,8 +146,7 @@ def imshow(image, ax=None, show_cbar=None, **kwargs):
         The `AxesImage` object returned by `plt.imshow`.
     """
     import matplotlib.pyplot as plt
-    if kwargs.get('cmap', None) == 'viridis':
-        kwargs['cmap'] = viridis
+
     lo, hi, cmap = _get_display_range(image)
 
     kwargs.setdefault('interpolation', 'nearest')
